//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  ATA/ATAPI-5 Host controller (OCIDEC-3)                     ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 8th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_controller.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_controller.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_controller (clk, nReset, rst, irq, IDEctrl_IDEen, IDEctrl_rst,TxbufQ, IDEctrl_ppen, IDEctrl_FATR0, IDEctrl_FATR1, a, d, we, PIO_cmdport_T1, PIO_cmdport_T2, PIO_cmdport_T4, PIO_cmdport_Teoc, PIO_cmdport_IORDYen, PIO_dport0_T1, PIO_dport0_T2, PIO_dport0_T4, PIO_dport0_Teoc, PIO_dport0_IORDYen, PIO_dport1_T1, PIO_dport1_T2, PIO_dport1_T4, PIO_dport1_Teoc, PIO_dport1_IORDYen, PIOsel, PIOack, PIOq, PIOtip, PIOpp_full, DMA_dev0_Td, DMA_dev0_Tm, DMA_dev0_Teoc, DMA_dev1_Td, DMA_dev1_Tm, DMA_dev1_Teoc, DMActrl_DMAen, DMActrl_dir, DMActrl_BeLeC0, DMActrl_BeLeC1, DMAsel, DMAack, DMAq, DMAtip, DMA_dmarq, DMATxFull, DMARxEmpty, DMA_req, DMA_ack, RESETn, DDi, DDo, DDoe, DA, CS0n, CS1n, DMARQ, DMACKn, DIORn, DIOWn, IORDY, INTRQ);
   parameter[15:0] TWIDTH = 8; 					// counter width
//--
   parameter[15:0] PIO_mode0_T1 = 6; 		// 70ns
   parameter[15:0] PIO_mode0_T2 = 28; 	// 290ns
   parameter[15:0] PIO_mode0_T4 = 2;		// 30ns
   parameter[15:0] PIO_mode0_Teoc = 23;	// 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
//--
   parameter[15:0] DMA_mode0_Tm = 4;		// 50ns
   parameter[15:0] DMA_mode0_Td = 21;		// 215ns
   parameter[15:0] DMA_mode0_Teoc  = 21;
//--   
   input 		clk; // master clock in
   input 		nReset; // asynchronous active low reset
   input 		rst; // synchronous active high reset
   output 	irq; // interrupt request signal
   reg 			irq;
//--   
   input 		IDEctrl_IDEen; // control register settings
   input 		IDEctrl_rst; // control register settings
   input 		IDEctrl_ppen; // control register settings
   input 		IDEctrl_FATR0; // control register settings
   input 		IDEctrl_FATR1; // control register settings
//--   
   input[3:0] a; 	// address input
   input[31:0] d; // data input
   input we; 			// write enable input '1'=write,'0'=read
//--   
   input[7:0] PIO_cmdport_T1;
   input[7:0] PIO_cmdport_T2;
   input[7:0] PIO_cmdport_T4;
   input[7:0] PIO_cmdport_Teoc;
   input 			PIO_cmdport_IORDYen; // PIO compatible timing settings
//--   
   input[7:0] PIO_dport0_T1;
   input[7:0] PIO_dport0_T2;
   input[7:0] PIO_dport0_T4;
   input[7:0] PIO_dport0_Teoc;
   input 			PIO_dport0_IORDYen; // PIO data-port device0 timing settings
//--      
   input[7:0] PIO_dport1_T1;
   input[7:0] PIO_dport1_T2;
   input[7:0] PIO_dport1_T4;
   input[7:0] PIO_dport1_Teoc;
   input 			PIO_dport1_IORDYen; // PIO data-port device1 timing settings
//--      
   input 			PIOsel; // PIO controller select
   output 		PIOack; // PIO controller acknowledge
//--   
   wire 				PIOack;
   output[15:0] PIOq; // PIO data out
   wire[15:0] 	PIOq;
   output 			PIOtip; // PIO transfer in progress
   reg 					PIOtip;
//--   
   output 			PIOpp_full; // PIO Write PingPong full
//--   
   wire 				PIOpp_full;
//--   
   input[7:0] 	DMA_dev0_Td; // DMA timing settings for device0
   input[7:0] 	DMA_dev0_Tm; // DMA timing settings for device0
   input[7:0] 	DMA_dev0_Teoc; // DMA timing settings for device0
   input[7:0] 	DMA_dev1_Td; // DMA timing settings for device1
   input[7:0] 	DMA_dev1_Tm; // DMA timing settings for device1
   input[7:0] 	DMA_dev1_Teoc; // DMA timing settings for device1
//--   
   input 				DMActrl_DMAen; // DMA settings
   input 				DMActrl_dir; // DMA settings
   input 				DMActrl_BeLeC0; // DMA settings
   input 				DMActrl_BeLeC1; // DMA settings
   input 				DMAsel; // DMA controller select
   output 			DMAack; // DMA controller acknowledge
//--   
   wire 				DMAack;
   output[31:0] DMAq; // DMA data out
   wire[31:0] DMAq;
   output DMAtip; // DMA transfer in progress
   reg DMAtip;
   output DMA_dmarq; // Synchronized ATA DMARQ line,output for status register
   wire DMA_dmarq;
   output DMATxFull; // DMA transmit buffer full
   wire DMATxFull;
   output DMARxEmpty; // DMA receive buffer empty
   wire DMARxEmpty;
   output DMA_req; // DMA request to external DMA engine
   wire DMA_req;
   input DMA_ack; // DMA acknowledge from external DMA engine
   output RESETn;
   reg RESETn;
   input[15:0] DDi;
   output[15:0] DDo;
   reg[15:0] DDo;
   output DDoe;
   reg DDoe;
   output[2:0] DA;
   reg[2:0] DA;
   output CS0n;
   reg CS0n;
   output CS1n;
   reg CS1n;
   input DMARQ;
   output DMACKn;
   reg DMACKn;
   output DIORn;
   reg DIORn;
   output DIOWn;
   reg DIOWn;
   input IORDY;
   input INTRQ;
output[31:0] TxbufQ;
   //
   // component declarations
   //
   // PIO mode 0 settings (@100MHz clock)
   // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
   // control / registers
   // PIO registers
   //
   // DMA mode 0 settings (@100MHz clock)
   // 215ns ==> T0 - Td - Tm = 480 - 50 - 215 = 215
   //
   //
   //
   //
   //
   //
   //
   //
   //
   // signals
   //
   wire SelDev; // selected device
   wire DMARxFull; // DMA receive buffer full
   // PIO / DMA signals
   reg PIOgo; // start PIO / DMA timing controller
   reg DMAgo; // start PIO / DMA timing controller
   wire PIOdone; // PIO / DMA timing controller done
   wire DMAdone; // PIO / DMA timing controller done
   // PIO signals
   wire PIOdior;
   wire PIOdiow;
   wire PIOoe;
   // PIO pingpong signals
   wire[15:0] PIOd;
   wire[3:0] PIOa;
   wire[31:0]TxbufQ;
   wire PIOreq;
   // DMA signals
   wire[15:0] DMAd;
   wire DMAdior;
   wire DMAdiow;
   // synchronized ATA inputs
   reg sDMARQ;
   reg sIORDY;

   //
   // synchronize incoming signals
   //
   reg cDMARQ; // capture DMARQ
   reg cIORDY; // capture IORDY
   reg cINTRQ; // capture INTRQ

   always @(posedge clk)
   begin
      cDMARQ <= DMARQ ;
      cIORDY <= IORDY ;
      cINTRQ <= INTRQ ;
      sDMARQ <= cDMARQ ;
      sIORDY <= cIORDY ;
      irq <= cINTRQ ;
   end
   assign DMA_dmarq = sDMARQ ;
   //
   // generate ATA signals
   //
   wire[15:0] iDDo;

   // generate registers for ATA signals
   always @(posedge clk or negedge nReset)
   begin : gen_regs
      if (nReset == 1'b0)
      begin
         RESETn <= 1'b0 ;
         DIORn <= 1'b1 ;
         DIOWn <= 1'b1 ;
         DA <= {3{1'b0}} ;
         CS0n <= 1'b1 ;
         CS1n <= 1'b1 ;
         DDo <= {16{1'b0}} ;
         DDoe <= 1'b0 ;
         DMACKn <= 1'b1 ;
      end
      else
      begin
         if (rst == 1'b1)
         begin
            RESETn <= 1'b0 ;
            DIORn <= 1'b1 ;
            DIOWn <= 1'b1 ;
            DA <= {3{1'b0}} ;
            CS0n <= 1'b1 ;
            CS1n <= 1'b1 ;
            DDo <= {16{1'b0}} ;
            DDoe <= 1'b0 ;
            DMACKn <= 1'b1 ;
         end
         else
         begin
            RESETn <= ~IDEctrl_rst ;
            DA <= PIOa[2:0] ;
            CS0n <= ~(~PIOa[3] & PIOtip) ; // CS0 asserted when A(3) = '0', negate during DMA transfers
            CS1n <= ~(PIOa[3] & PIOtip) ; // CS1 asserted when A(3) = '1', negate during DMA transfers
            if (PIOtip == 1'b1)
            begin
               DDo <= PIOd ;
               DDoe <= PIOoe ;
               DIORn <= ~PIOdior ;
               DIOWn <= ~PIOdiow ;
            end
            else
            begin
               DDo <= DMAd ;
               DDoe <= DMActrl_dir & DMAtip ;
               DIORn <= ~DMAdior ;
               DIOWn <= ~DMAdiow ;
            end
            DMACKn <= ~DMAtip ;
         end
      end
   end
   //
   // generate bus controller statemachine
   //
   parameter[2:0] idle = 1;
   parameter[2:0] PIO_state = 2;
   parameter[2:0] DMA_state = 4;
   reg[2:0] nxt_state; // next_state, current_state
   reg[2:0] c_state; // next_state, current_state
   reg iPIOgo;
   reg iDMAgo;

   // generate next state decoder + output decoder
   always @(c_state or DMActrl_DMAen or DMActrl_dir or PIOreq or sDMARQ or
            DMATxFull or DMARxFull or PIOdone or DMAdone)
   begin : gen_nxt_state
      nxt_state = c_state ; // initialy stay in current state
      iPIOgo = 1'b0 ;
      iDMAgo = 1'b0 ;
      case (c_state)
         // idle
         idle :
         begin
            if ((sDMARQ == 1'b1) & (DMActrl_DMAen == 1'b1))
            begin
               // DMA transfer pending ?
               if (((DMActrl_dir == 1'b1) & (DMATxFull == 1'b1)) | ((DMActrl_dir == 1'b0) & (DMARxFull == 1'b0)))
               begin
                   nxt_state = DMA_state ; // DMA transfer
                   iDMAgo = 1'b1 ; // start DMA timing controller
               end
            end
            // PIO transfer pending ?
            else if (PIOreq == 1'b1)
            begin
                nxt_state = PIO_state ; // PIO transfer
                iPIOgo = 1'b1 ;
            end
         end
         // PIO transfer
         PIO_state :
         begin
             if (PIOdone == 1'b1)
             begin
               	 nxt_state = idle ;
             end
         end
         // DMA transfer
         DMA_state :
         begin
             if (DMAdone == 1'b1)
             begin
                 nxt_state = idle ;
             end
         end
         default :
         begin
             nxt_state = idle ; // go to idle state
         end
      endcase
   end

   // generate registers
   always @(posedge clk or negedge nReset)
   begin : gen_regs_
      if (nReset == 1'b0)
      begin
         c_state <= idle ;
         PIOgo <= 1'b0 ;
         DMAgo <= 1'b0 ;
      end
      else
      begin
         if (rst == 1'b1)
         begin
            c_state <= idle ;
            PIOgo <= 1'b0 ;
            DMAgo <= 1'b0 ;
         end
         else
         begin
            c_state <= nxt_state ;
            PIOgo <= iPIOgo ;
            DMAgo <= iDMAgo ;
         end
      end
   end

   // generate PIOtip / DMAtip
   always @(posedge clk or negedge nReset)
   begin : gen_tip
      if (nReset == 1'b0)
      begin
         PIOtip <= 1'b0 ;
         DMAtip <= 1'b0 ;
      end
      else
      begin
         if (rst == 1'b1)
         begin
            PIOtip <= 1'b0 ;
            DMAtip <= 1'b0 ;
         end
         else
         begin
            PIOtip <= iPIOgo | (PIOtip & ~PIOdone) ;
            DMAtip <= iDMAgo | (DMAtip & ~((DMAdone & DMActrl_dir) | (DMAdone & ~sDMARQ & ~DMActrl_dir))) ;
         end
      end
   end
   //
   // Hookup PIO controller
   //


   atahost_pio_controller #(TWIDTH,
PIO_mode0_T1,
PIO_mode0_T2,
PIO_mode0_T4,
PIO_mode0_Teoc
) PIO_control(
      .clk(clk),
      .nReset(nReset),
      .rst(rst),
      .IDEctrl_IDEen(IDEctrl_IDEen),
      .IDEctrl_ppen(IDEctrl_ppen),
      .IDEctrl_FATR0(IDEctrl_FATR0),
      .IDEctrl_FATR1(IDEctrl_FATR1),
      .cmdport_T1(PIO_cmdport_T1),
      .cmdport_T2(PIO_cmdport_T2),
      .cmdport_T4(PIO_cmdport_T4),
      .cmdport_Teoc(PIO_cmdport_Teoc),
      .cmdport_IORDYen(PIO_cmdport_IORDYen),
      .dport0_T1(PIO_dport0_T1),
      .dport0_T2(PIO_dport0_T2),
      .dport0_T4(PIO_dport0_T4),
      .dport0_Teoc(PIO_dport0_Teoc),
      .dport0_IORDYen(PIO_dport0_IORDYen),
      .dport1_T1(PIO_dport1_T1),
      .dport1_T2(PIO_dport1_T2),
      .dport1_T4(PIO_dport1_T4),
      .dport1_Teoc(PIO_dport1_Teoc),
      .dport1_IORDYen(PIO_dport1_IORDYen),
      .sel(PIOsel),
      .ack(PIOack),
      .a(a),
      .we(we),
      .d(d[15:0]),
      .q(PIOq),
      .PIOreq(PIOreq),
      .PPFull(PIOpp_full),
      .go(PIOgo),
      .done(PIOdone),
      .PIOa(PIOa),
      .PIOd(PIOd),
      .SelDev(SelDev),  //SelDev
      .DDi(DDi),
      .DDoe(PIOoe),
      .DIOR(PIOdior),
      .DIOW(PIOdiow),
      .IORDY(sIORDY)
   );
   //
   // Hookup DMA access controller
   //


   atahost_dma_actrl #(TWIDTH,
DMA_mode0_Tm,
DMA_mode0_Td,
DMA_mode0_Teoc
) DMA_control(
      .clk(clk),
      .nReset(nReset),
      .rst(rst),
      .IDEctrl_rst(IDEctrl_rst),
      .DMActrl_DMAen(DMActrl_DMAen),
      .DMActrl_dir(DMActrl_dir),
      .DMActrl_BeLeC0(DMActrl_BeLeC0),
      .DMActrl_BeLeC1(DMActrl_BeLeC1),
      .dev0_Td(DMA_dev0_Td),
      .dev0_Tm(DMA_dev0_Tm),
      .dev0_Teoc(DMA_dev0_Teoc),
      .dev1_Td(DMA_dev1_Td),
      .dev1_Tm(DMA_dev1_Tm),
      .dev1_Teoc(DMA_dev1_Teoc),
      .sel(DMAsel),
      .ack(DMAack),
      .we(we),
      .TxD(d), 
      .TxbufQ(TxbufQ),
      .TxFull(DMATxFull),
      .RxQ(DMAq),
      .RxFull(DMARxFull),
      .RxEmpty(DMARxEmpty),
      .DMA_req(DMA_req),
      .DMA_ack(DMA_ack),
      .SelDev(SelDev),  //SelDev
      .Go(DMAgo),
      .Done(DMAdone),
      .DDi(DDi),
      .DDo(DMAd),
      .DIOR(DMAdior),
      .DIOW(DMAdiow),
      .DMARQ(sDMARQ)
   );
endmodule
