//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  DMA (single- and multiword) mode access controller         ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 9th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_dma_actrl.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_dma_actrl.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
// Host accesses to DMA ports are 32bit wide. Accesses are made by 2 consecutive 16bit accesses to the ATA
// device's DataPort. The MSB HostData(31:16) is transfered first, then the LSB HostData(15:0) is transfered.
//
//-------------------------
// DMA Access Controller --
//-------------------------
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_dma_actrl (clk, nReset, rst, IDEctrl_rst, sel, we, ack, dev0_Tm, dev0_Td, dev0_Teoc,TxbufQ, dev1_Tm, dev1_Td, dev1_Teoc, DMActrl_DMAen, DMActrl_dir, DMActrl_BeLeC0, DMActrl_BeLeC1, TxD, TxFull, RxQ, RxEmpty, RxFull, DMA_req, DMA_ack, DMARQ, SelDev, Go, Done, DDi, DDo, DIOR, DIOW);

    // counter width
   parameter TWIDTH = 8;
    // 50ns
   parameter DMA_mode0_Tm = 4;
    // 215ns
   parameter DMA_mode0_Td = 21;
   parameter DMA_mode0_Teoc  = 21;
   input clk; // master clock
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input IDEctrl_rst; // IDE control register bit0, 'rst'
   input sel; // DMA buffers selected
   input we; // write enable input
   output ack; // acknowledge output
   wire ack;
   input[7:0] dev0_Tm; // DMA mode timing device 0
   input[7:0] dev0_Td; // DMA mode timing device 0
   input[7:0] dev0_Teoc; // DMA mode timing device 0
   input[7:0] dev1_Tm; // DMA mode timing device 1
   input[7:0] dev1_Td; // DMA mode timing device 1
   input[7:0] dev1_Teoc; // DMA mode timing device 1
   input DMActrl_DMAen; // control register settings
   input DMActrl_dir; // control register settings
   input DMActrl_BeLeC0; // control register settings
   input DMActrl_BeLeC1; // control register settings
   input[31:0] TxD; // DMA transmit data
   output TxFull; // DMA transmit buffer full
   wire TxFull;
   output[31:0] RxQ; // DMA receive data
   wire[31:0] RxQ;
   output RxEmpty; // DMA receive buffer empty
   wire RxEmpty;
   output RxFull; // DMA receive buffer full
   wire RxFull;
   output DMA_req; // DMA request to external DMA engine
   wire DMA_req;
   input DMA_ack; // DMA acknowledge from external DMA engine
   input DMARQ; // ATA devices request DMA transfer
   input SelDev; // Selected device
   input Go; // Start transfer sequence
   output Done; // Transfer sequence done
   wire Done;
   input[15:0] DDi; // Data from ATA DD bus
   output[15:0] DDo; // Data towards ATA DD bus
   wire[15:0] DDo;
   output DIOR; 
   wire DIOR;
   output DIOW; 
   wire DIOW;
output[31:0]TxbufQ;
   //
   // component declarations
   //
   // DMA mode 0 settings (@100MHz clock)
   // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
   // timing register settings
   // control signals
   // return signals
   // ATA signals
   // IOread signal, active high
   // IOwrite signal, active high
   //
   // data width
   // data output
   wire Tdone; 
   reg Tfw; 
   reg RxWr; 
   reg TxRd; 
   wire dstrb; 
   wire rd_dstrb; 
   wire wr_dstrb; 
   wire[31:0] TxbufQ; 
   wire[31:0] RxbufD; 

   // note: *fw = *first_word, *lw = *last_word
   //
   // generate DDi/DDo controls
   //
   reg[15:0] writeDfw; 
   reg[15:0] writeDlw; 
   reg[15:0] readDfw; 
   reg[15:0] readDlw; 
   wire BeLeC; // BigEndian <-> LittleEndian conversion
   // generate byte_swap signal
   assign BeLeC = (~SelDev & DMActrl_BeLeC0) | (SelDev & DMActrl_BeLeC1) ;

   // generate Tfw (Transfering first word)
   always @(posedge clk or negedge nReset)
   begin : gen_Tfw
      if (nReset == 1'b0)
      begin
         Tfw <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            Tfw <= 1'b0 ; 
         end
         else
         begin
            Tfw <= Go | (Tfw & ~Tdone) ; 
         end 
      end 
   end 

   // transmit data part
   always @(posedge clk)
   begin : gen_writed_pipe
      if (TxRd == 1'b1)
      begin
         // reload registers
         if (BeLeC == 1'b1)
         begin
            // Do big<->little endian conversion
            writeDfw[15:8] <= TxbufQ[7:0] ; // TxbufQ = data from transmit buffer
            writeDfw[7:0] <= TxbufQ[15:8] ; 
            writeDlw[15:8] <= TxbufQ[23:16] ; 
            writeDlw[7:0] <= TxbufQ[31:24] ; 
         end
         else
         begin
            // don't do big<->little endian conversion
            writeDfw <= TxbufQ[31:16] ; 
            writeDlw <= TxbufQ[15:0] ; 
         end 
      end
      else if (wr_dstrb == 1'b1)
      begin
         // next word to transfer
         writeDfw <= writeDlw ; 
      end  
   end 
   assign DDo = writeDfw ;

   // generate transmit register read request
   always @(posedge clk or negedge nReset)
   begin : gen_Tx_rreq
      if (nReset == 1'b0)
      begin
         TxRd <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            TxRd <= 1'b0 ; 
         end
         else
         begin
            TxRd <= Go & DMActrl_dir ; 
         end 
      end 
   end 

   // receive
   always @(posedge clk)
   begin : gen_readd_pipe
      if (rd_dstrb == 1'b1)
      begin
         readDfw <= readDlw ; // shift previous read word to msb
         if (BeLeC == 1'b1)
         begin
            // swap bytes
            readDlw[15:8] <= DDi[7:0] ; 
            readDlw[7:0] <= DDi[15:8] ; 
         end
         else
         begin
            // don't swap bytes
            readDlw <= DDi ; 
         end 
      end  
   end 
   // RxD = data to receive buffer
   assign RxbufD = (BeLeC == 1'b0) ? ({readDfw, readDlw}) : ({readDlw, readDfw}) ;

   // generate receive register write request
   always @(posedge clk or negedge nReset)
   begin : gen_Rx_wreq
      if (nReset == 1'b0)
      begin
         RxWr <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            RxWr <= 1'b0 ; 
         end
         else
         begin
            RxWr <= ~Tfw & rd_dstrb ; 
         end 
      end 
   end  
   //
   // Hookup DMA read / write buffers
   //
   wire DMArst; 
   reg RxRd; 
   reg TxWr;
   reg[31:0] TxD_reg; 
   wire iRxEmpty; 
   // generate DMA reset signal
   assign DMArst = rst | IDEctrl_rst ;


   atahost_reg_buf #(32) Txbuf(
      .clk(clk), 
      .nReset(nReset), 
      .rst(DMArst), 
      .D(TxD_reg), 
      .Q(TxbufQ), 
      .rd(TxRd), 
      .wr(TxWr), 
      .valid(TxFull)
   ); 

 wire[31:0] RxQ1;
   atahost_fifo #(7,32) Rxbuf(
      .clk(clk), 
      .nReset(nReset), 
      .rst(DMArst), 
      .D(RxbufD), 
      .Q(RxQ1), 
      .rreq(RxRd), 
      .wreq(RxWr), 
      .empty(iRxEmpty), 
      .full(RxFull)
   ); 
   assign RxEmpty = iRxEmpty ;
   //
   // generate DMA buffer access signals
   //
   //assign RxRd = sel & ~we & ~RxEmpty ;
   always @(posedge clk)
   begin
       if( ~RxEmpty)
       begin
           RxRd <= sel & (~we);
       end
       else
       begin
       		 RxRd <= 1'b0;
       end
   end
   reg[31:0] RxQ_reg;
   always @(posedge clk)
   begin
   		if( RxRd)
   		begin
   				 RxQ_reg <= RxQ1;
   		end
   		else
   		;
   end
   assign RxQ=RxQ_reg;
   //
   always @( posedge clk )
   		begin
   				if( ~TxFull)
   				begin
   				    TxWr <= sel & we;
   				    TxD_reg	<= TxD;
   				end
   				else
   				begin
   				    TxWr <= 1'b0;
   				end
   		end
   //assign TxWr = sel & we & ~TxFull ;
   assign ack = RxRd | TxWr ;
   //
   // generate request signal for external DMA engine
   //
   reg hgo; 
   reg iDMA_req; 
   wire request; 

   // generate hold-go
   always @(posedge clk or negedge nReset)
   begin : gen_hgo
      if (nReset == 1'b0)
      begin
         hgo <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            hgo <= 1'b0 ; 
         end
         else
         begin
            hgo <= Go | (hgo & ~(wr_dstrb & ~Tfw) & DMActrl_dir) ; 
         end 
      end 
   end 
   assign request = (DMActrl_dir & DMARQ & ~TxFull & ~hgo) | ~RxEmpty ;

   always @(posedge clk or negedge nReset)
   begin
      if (nReset == 1'b0)
      begin
         iDMA_req <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            iDMA_req <= 1'b0 ; 
         end
         else
         begin
            iDMA_req <= DMActrl_DMAen & ~DMA_ack & (request | iDMA_req) ; 
         end
         //				DMA_req <= (DMActrl_DMAen and DMActrl_dir and DMARQ and not TxFull and not hgo) or not RxEmpty; 
      end 
   end 
   assign DMA_req = iDMA_req ;
   //
   // DMA timing controller
   //
   reg[TWIDTH - 1:0] Tm; 
   reg[TWIDTH - 1:0] Td; 
   reg[TWIDTH - 1:0] Teoc; 
   wire[TWIDTH - 1:0] Tdmack_ext; 
   reg dTfw; 
   reg igo; 

   //
   // generate internal GO signal
   //
   always @(posedge clk or negedge nReset)
   begin : gen_igo
      if (nReset == 1'b0)
      begin
         igo <= 1'b0 ; 
         dTfw <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            igo <= 1'b0 ; 
            dTfw <= 1'b0 ; 
         end
         else
         begin
            igo <= Go | (~Tfw & dTfw) ; 
            dTfw <= Tfw ; 
         end 
      end 
   end 

   //
   // select timing settings for the addressed device
   //
   always @(posedge clk)
   begin : sel_dev_t
      if (SelDev == 1'b1)
      begin
         // device1 selected
         Tm <= dev1_Tm ; 
         Td <= dev1_Td ; 
         Teoc <= dev1_Teoc ; 
      end
      else
      begin
         // device0 selected
         Tm <= dev0_Tm ; 
         Td <= dev0_Td ; 
         Teoc <= dev0_Teoc ; 
      end  
   end 
   //
   // hookup timing controller
   //

 
   atahost_dma_tctrl #(TWIDTH,       
DMA_mode0_Tm, 
DMA_mode0_Td, 
DMA_mode0_Teoc
) DMA_timing_ctrl(
      .clk(clk),       
      .nReset(nReset), 
      .rst(rst),       
      .Tm(Tm), 
      .Td(Td), 
      .Teoc(Teoc), 
      .go(igo), 
      .we(DMActrl_dir), 
      .done(Tdone), 
      .dstrb(dstrb), 
      .DIOR(DIOR), 
      .DIOW(DIOW)
   ); 
   assign Done = Tdone & ~Tfw ;
   assign rd_dstrb = dstrb & ~DMActrl_dir ;
   assign wr_dstrb = dstrb & DMActrl_dir ;
endmodule
