//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//
// component declarations
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  synchronous single clock fifo, uses LFSR pointers          ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 12th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_fifo.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_fifo.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
`define ADEPTH 3
module atahost_fifo (clk, nReset, rst, rreq, wreq, empty, full, D, Q);

    // fifo depth, this must be a number according to the following range
   parameter DEPTH = 31;
   parameter SIZE  = 32;
   input clk; // master clock in
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset    	
   input rreq; // read request
   input wreq; // write request    	
   output empty; // fifo empty
   wire empty;
   output full; // fifo full    	
   wire full;
   input[SIZE - 1:0] D; // data input
   output[SIZE - 1:0] Q; 
   wire[SIZE - 1:0] Q;

   //
   // function declarations
   //
 
   reg[SIZE - 1:0] mem[DEPTH - 1:0]; //
   wire[`ADEPTH - 1:0] wr_ptr; 
   wire[`ADEPTH - 1:0] rd_ptr; 
   wire[`ADEPTH - 1:0] dwr_ptr; 
   wire[`ADEPTH - 1:0] drd_ptr; 

 
   atahost_lfsr #(`ADEPTH,0) wr_ptr_lfsr(
      .clk(clk), 
      .ena(wreq), 
      .nReset(nReset), 
      .rst(rst), 
      .Q(wr_ptr), 
      .Qprev(dwr_ptr)
   ); 

  
   atahost_lfsr #(`ADEPTH,0) rd_ptr_lfsr(
      .clk(clk), 
      .ena(rreq), 
      .nReset(nReset), 
      .rst(rst), 
      .Q(rd_ptr), 
      .Qprev(drd_ptr)
   ); 
   assign full = (wr_ptr == drd_ptr) ? 1'b1 : 1'b0 ;
   assign empty = (rd_ptr == wr_ptr) ? 1'b1 : 1'b0 ;

   always @(posedge clk)
   begin : gen_mem
      // generate memory structure
      if (wreq == 1'b1)
      begin
         mem[wr_ptr] <= D ; 
      end  
   end 
   reg[31:0] Q_reg;
   always @(posedge clk)
           Q_reg   <= mem[rd_ptr];
   assign Q = Q_reg ;
endmodule
