//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  Linear Feedback Shift Register                             ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 12th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_lfsr.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_lfsr.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_lfsr(
				clk,
				ena,
				nReset,
				rst,
				Q,
				Qprev
				);
//--
parameter[3:0] 	TAPS  = 8;
parameter 			OFFSET  = 0;
input 					clk; 					// clock input
input 					ena; 					// count enable
input 					nReset; 			// asynchronous active low reset
input 					rst; 					// synchronous active high reset
output[TAPS:1] 	Q; 						// count value
wire[TAPS:1] 		Q;
output[TAPS:1] 	Qprev;
wire[TAPS:1] 		Qprev;
//--
function [0:0] 	lsb;
input[3:0] 			tap;
input[TAPS:1] 	Q;
begin
    case (tap)
     3:begin lsb = ~(Q[3] ^ Q[2]);end
    // 4:begin lsb = ~(Q[4] ^ Q[3]);end
		// 5:begin lsb = ~(Q[5] ^ Q[3]);end
    // 6:begin lsb = ~(Q[6] ^ Q[5]);end
    // 7:begin lsb = ~(Q[7] ^ Q[6]);end
    // 8:begin lsb = ~((~(Q[8] ^ Q[6])) ^ (~(Q[5] ^ Q[4])));end
    // 9:begin lsb = ~(Q[9] ^ Q[5]);end
    //10:begin lsb = ~(Q[10] ^ Q[7]);end
    //11:begin lsb = ~(Q[11] ^ Q[9]);end
    //12:begin lsb = ~((~(Q[12] ^ Q[6])) ^ (~(Q[4] ^ Q[1])));end
    //13:begin lsb = ~((~(Q[13] ^ Q[4])) ^ (~(Q[3] ^ Q[1])));end
    //14:begin lsb = ~((~(Q[14] ^ Q[5])) ^ (~(Q[3] ^ Q[1])));end
    //15:begin lsb = ~(Q[15] ^ Q[14]);end
    //16:begin lsb = ~((~(Q[16] ^ Q[15])) ^ (~(Q[13] ^ Q[4])));end
    endcase
end
endfunction
//--
reg msb;
reg[TAPS:1] iQ;
//reg[TAPS:1] tmpQ;
//reg tmpmsb;
//integer n;
always @(posedge clk or negedge nReset)
begin : gen_regs
    if (nReset == 1'b0)
    begin
        iQ <= 3'd0 ;
        msb <= 1'b1 ;
    end
    else
    begin
       	if (rst == 1'b1)
       	begin
          	iQ <= 3'd0  ;
          	msb <= 1'b1 ;
       	end
       	else if (ena == 1'b1)
       	begin
          	iQ <= ({iQ[TAPS - 1:1], lsb(TAPS, iQ)}) ;
          	msb <= iQ[TAPS] ;
       	end
    end
end
//assign outputs
assign Q 			= iQ ;
assign Qprev 	= ({msb, iQ[TAPS:2]}) ;
endmodule
