//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  PIO Access Controller (common for OCIDEC 2 and above)      ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ---
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 9th, 2001
// rev.: 1.0a april 12th, 2001 Removed references to records.vhd
//
//
//  CVS Log
//
//  $Id: atahost_pio_actrl.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_pio_actrl.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//-------------------------
// PIO Access controller --
//-------------------------
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_pio_actrl (clk, nReset, rst, IDEctrl_FATR0, IDEctrl_FATR1, cmdport_T1, cmdport_T2, cmdport_T4, cmdport_Teoc, cmdport_IORDYen, dport0_T1, dport0_T2, dport0_T4, dport0_Teoc, dport0_IORDYen, dport1_T1, dport1_T2, dport1_T4, dport1_Teoc, dport1_IORDYen, SelDev, go, done, dir, a, q, DDi, oe, DIOR, DIOW, IORDY);

    // counter width
   parameter TWIDTH = 8;
    // 70ns
   parameter PIO_mode0_T1 = 6;
    // 290ns
   parameter PIO_mode0_T2 = 28;
    // 30ns
   parameter PIO_mode0_T4 = 2;
   parameter PIO_mode0_Teoc  = 23;
   input clk; // master clock
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input IDEctrl_FATR0; 
   input IDEctrl_FATR1; 
   input[7:0] cmdport_T1; 
   input[7:0] cmdport_T2; 
   input[7:0] cmdport_T4; 
   input[7:0] cmdport_Teoc; 
   input cmdport_IORDYen; // PIO command port / non-fast timing
   input[7:0] dport0_T1; 
   input[7:0] dport0_T2; 
   input[7:0] dport0_T4; 
   input[7:0] dport0_Teoc; 
   input dport0_IORDYen; // PIO mode data-port / fast timing device 0
   input[7:0] dport1_T1; 
   input[7:0] dport1_T2; 
   input[7:0] dport1_T4; 
   input[7:0] dport1_Teoc; 
   input dport1_IORDYen; // PIO mode data-port / fast timing device 1
   input SelDev; // Selected device	
   input go; // Start transfer sequence
   output done; // Transfer sequence done
   wire done;
   input dir; // Transfer direction '1'=write, '0'=read
   input[3:0] a; // PIO transfer address
   output[15:0] q; // Data read from ATA devices
   reg[15:0] q;
   input[15:0] DDi; // Data from ATA DD bus
   output oe; // DDbus output-enable signal
   wire oe;
   output DIOR; 
   wire DIOR;
   output DIOW; 
   wire DIOW;
   input IORDY; 

   //
   // Component declarations
   //
   // PIO mode 0 settings (@100MHz clock)
   // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
   // timing/control register settings
   // control signals
   // return signals
   // ATA signals
   // IOread signal, active high
   // IORDY signal
   //
   wire dstrb; 
   reg[TWIDTH - 1:0] T1; 
   reg[TWIDTH - 1:0] T2; 
   reg[TWIDTH - 1:0] T4; 
   reg[TWIDTH - 1:0] Teoc; 
   reg IORDYen; 

   //
   //------------------------
   // PIO transfer control --
   //------------------------
   //
   // capture ATA data for PIO access
   always @(posedge clk)
   begin : gen_PIOq
      if (dstrb == 1'b1)
      begin
         q <= DDi ; 
      end  
   end 

 wire Asel; // address selected
 
 assign Asel = ~a[3] & ~a[2] & ~a[1] & ~a[0]; // data port
      reg [TWIDTH - 1:0] iT1; 
      reg [TWIDTH - 1:0] iT2; 
      reg [TWIDTH - 1:0] iT4; 
      reg [TWIDTH - 1:0] iTeoc; 
      reg  iIORDYen; 

   //
   // PIO timing controllers
   //
   // select timing settings for the addressed port
   always @(clk or a or SelDev or IDEctrl_FATR1 or IDEctrl_FATR0 or cmdport_T1 or 
            cmdport_T2 or cmdport_T4 or cmdport_Teoc or cmdport_IORDYen or dport0_T1 or 
            dport0_T2 or dport0_T4 or dport0_Teoc or dport0_IORDYen or dport1_T1 or 
            dport1_T2 or dport1_T4 or dport1_Teoc or dport1_IORDYen)
   begin : sel_port_t
     
      // initially set timing registers to compatible timing
      iT1 = cmdport_T1; 
      iT2 = cmdport_T2; 
      iT4 = cmdport_T4; 
      iTeoc = cmdport_Teoc; 
      iIORDYen = cmdport_IORDYen; 
      // detect data-port access
      
      if (Asel == 1'b1)
      begin
         // data port selected, 16bit transfers
         if ((SelDev == 1'b1) & (IDEctrl_FATR1 == 1'b1))
         begin
            // data port1 selected and enabled ?
            iT1 = dport1_T1; 
            iT2 = dport1_T2; 
            iT4 = dport1_T4; 
            iTeoc = dport1_Teoc; 
            iIORDYen = dport1_IORDYen; 
         end
         else if ((SelDev == 1'b0) & (IDEctrl_FATR0 == 1'b1))
         begin
            // data port0 selected and enabled ?
            iT1 = dport0_T1; 
            iT2 = dport0_T2; 
            iT4 = dport0_T4; 
            iTeoc = dport0_Teoc; 
            iIORDYen = dport0_IORDYen; 
         end 
      end 
     
       
   end 
   always @( posedge clk)
   begin
   		 T1 <= iT1 ; 
      T2 <= iT2 ; 
      T4 <= iT4 ; 
      Teoc <= iTeoc ; 
      IORDYen <= iIORDYen ; 
   end
   
   //
   // hookup timing controller
   //


   atahost_pio_tctrl #(TWIDTH,       
PIO_mode0_T1, 
PIO_mode0_T2, 
PIO_mode0_T4, 
PIO_mode0_Teoc
) PIO_timing_controller(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .IORDY_en(IORDYen), 
      .T1(T1), 
      .T2(T2), 
      .T4(T4), 
      .Teoc(Teoc), 
      .go(go), 
      .we(dir), 
      .oe(oe), 
      .done(done), 
      .dstrb(dstrb), 
      .DIOR(DIOR), 
      .DIOW(DIOW), 
      .IORDY(IORDY)
   ); 
endmodule
