//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
//
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  ATA/ATAPI-5 PIO controller with write PingPong             ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march 8th, 2001. Initial release
//
//  CVS Log
//
//  $Id: atahost_pio_controller.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_pio_controller.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_pio_controller (clk, nReset, rst, IDEctrl_IDEen, IDEctrl_ppen, IDEctrl_FATR0, IDEctrl_FATR1, cmdport_T1, cmdport_T2, cmdport_T4, cmdport_Teoc, cmdport_IORDYen, dport0_T1, dport0_T2, dport0_T4, dport0_Teoc, dport0_IORDYen, dport1_T1, dport1_T2, dport1_T4, dport1_Teoc, dport1_IORDYen, sel, ack, a, we, d, q, PIOreq, PPFull, go, done, PIOa, PIOd, SelDev, DDi, DDoe, DIOR, DIOW, IORDY);

    // counter width
   parameter TWIDTH = 8;
    // 70ns
   parameter PIO_mode0_T1 = 6;
    // 290ns
   parameter PIO_mode0_T2 = 28;
    // 30ns
   parameter PIO_mode0_T4 = 2;
   parameter PIO_mode0_Teoc  = 23;
   input clk; // master clock in
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input IDEctrl_IDEen; 
   input IDEctrl_ppen; 
   input IDEctrl_FATR0; 
   input IDEctrl_FATR1; 
   input[7:0] cmdport_T1; 
   input[7:0] cmdport_T2; 
   input[7:0] cmdport_T4; 
   input[7:0] cmdport_Teoc; 
   input cmdport_IORDYen; // PIO command port / non-fast timing
   input[7:0] dport0_T1; 
   input[7:0] dport0_T2; 
   input[7:0] dport0_T4; 
   input[7:0] dport0_Teoc; 
   input dport0_IORDYen; // PIO mode data-port / fast timing device 0
   input[7:0] dport1_T1; 
   input[7:0] dport1_T2; 
   input[7:0] dport1_T4; 
   input[7:0] dport1_Teoc; 
   input dport1_IORDYen; // PIO mode data-port / fast timing device 1
   input sel; // PIO controller selected
   output ack; // PIO controller acknowledge
   wire ack;
   input[3:0] a; // lower address bits
   input we; // write enable input
   input[15:0] d; 
   output[15:0] q; 
   wire[15:0] q;
   output PIOreq; // PIO transfer request
   wire PIOreq;
   output PPFull; // PIO Write PingPong Full
   wire PPFull;
   input go; // start PIO transfer
   output done; // done with PIO transfer
   wire done;
   output[3:0] PIOa; // PIO address, address lines towards ATA devices
   wire[3:0] PIOa;
   output[15:0] PIOd; // PIO data, data towards ATA devices
   wire[15:0] PIOd;
   output SelDev; // Selected Device, Dev-bit in ATA Device/Head register
   reg SelDev;
   input[15:0] DDi; 
   output DDoe; 
   wire DDoe;
   output DIOR; 
   wire DIOR;
   output DIOW; 
   wire DIOW;
   input IORDY; 

   //
   // component declarations
   //
   // PIO mode 0 settings (@100MHz clock)
   // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
   //
   // signals
   //
   // PIO pingpong signals
   wire[15:0] pp_d; 
   wire[3:0] pp_a; 
   wire pp_we; 
   wire idone; 
	 wire Asel=~pp_a[3] & pp_a[2] & pp_a[1] & ~pp_a[0];
   //
   // generate selected device
   //
   always @(posedge clk)
   begin : gen_seldev
       // address selected
      //Asel = ~pp_a[3] & pp_a[2] & pp_a[1] & ~pp_a[0]; // header/device register "0110"  drive or head register
      if ((idone == 1'b1) & (Asel == 1'b1) & (pp_we == 1'b1))
      begin
         SelDev <= pp_d[4] ; 
      end  
   end 
   //
   // generate PIO write pingpong system
   //
   reg[15:0] ping_d; 
   reg[15:0] pong_d; 
   reg[3:0] ping_a; 
   reg[3:0] pong_a; 
   reg ping_we; 
   reg pong_we; 
   reg ping_valid; 
   reg pong_valid; 
   reg dping_valid; 
   reg dpong_valid; 
   reg wpp; 
   reg rpp; 
   reg dsel; 
   wire sel_strb; 
   reg iack; 


wire ping_re; 
    wire ping_fe; 
      wire pong_re; 
      wire pong_fe; 
      // detect rising edge of ping_valid and pong_valid
    assign  ping_re = ping_valid & ~dping_valid & we; 
    assign   pong_re = pong_valid & ~dpong_valid & we; 
      // detect falling edge of ping_valid and pong_valid
    assign   ping_fe = ~ping_valid & dping_valid;              
    assign    pong_fe = ~pong_valid & dpong_valid; 
      //
   // generate PIO acknowledge
   always @(posedge clk)
   begin : gen_ack
     // reg ping_re; 
     // reg ping_fe; 
     // reg pong_re; 
     // reg pong_fe; 
     // // detect rising edge of ping_valid and pong_valid
     // ping_re = ping_valid & ~dping_valid & we; 
     // pong_re = pong_valid & ~dpong_valid & we; 
     // // detect falling edge of ping_valid and pong_valid
     // ping_fe = ~ping_valid & dping_valid;              
     // pong_fe = ~pong_valid & dpong_valid;              
      if ((pp_we == 1'b1) & (IDEctrl_ppen == 1'b1))
      begin               
         // write sequence
         if (wpp == 1'b1)
         begin
            iack <= ping_re ; 
         end
         else
         begin
            iack <= pong_re ; 
         end 
      end
      else
      begin
         // read sequence
         if (rpp == 1'b1)
         begin
            iack <= ping_fe ; 
         end
         else
         begin
            iack <= pong_fe ; 
         end 
      end  
   end 
   assign ack = (iack | ~IDEctrl_IDEen) & sel ;

   // generate select-strobe, hold sel_strb until pingpong system ready for new data
   always @(posedge clk or negedge nReset)
   begin : gen_sel_strb
      if (nReset == 1'b0)
      begin
         dsel <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            dsel <= 1'b0 ; 
         end
         else
         begin
            dsel <= sel_strb | (dsel & sel) ; 
         end 
      end 
   end 
   //
   assign sel_strb = sel & ~dsel & IDEctrl_IDEen & ((wpp & ~ping_valid) | (~wpp & ~pong_valid)) ;

   // generate pingpong control
   always @(posedge clk or negedge nReset)
   begin : gen_pp
      if (nReset == 1'b0)
      begin
         wpp <= 1'b0 ; 
         rpp <= 1'b0 ; 
         ping_valid <= 1'b0 ; 
         pong_valid <= 1'b0 ; 
         dping_valid <= 1'b0 ; 
         dpong_valid <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            wpp <= 1'b0 ; 
            rpp <= 1'b0 ; 
            ping_valid <= 1'b0 ; 
            pong_valid <= 1'b0 ; 
            dping_valid <= 1'b0 ; 
            dpong_valid <= 1'b0 ; 
         end
         else
         begin
            wpp <= (wpp ^ (iack & we)) & IDEctrl_ppen ; 
            rpp <= (rpp ^ (idone & pp_we)) & IDEctrl_ppen ; 
            ping_valid <= ((wpp & sel_strb) | ping_valid) & ~(rpp & idone) ; 
            pong_valid <= ((~wpp & sel_strb) | pong_valid) & ~(~rpp & idone) ; 
            dping_valid <= ping_valid ; 
            dpong_valid <= pong_valid ; 
         end 
      end 
   end 
   // generate pingpong full signal
   assign PPFull = (IDEctrl_ppen == 1'b1) ? (ping_valid & pong_valid) : pong_valid ;

   // fill ping/pong registers
   always @(posedge clk)
   begin : fill_pp
      if (sel == 1'b1)
      begin
         if (wpp == 1'b1)
         begin
            if (ping_valid == 1'b0)
            begin
               ping_d <= d ; 
               ping_a <= a ; 
               ping_we <= we ; 
            end 
         end
         else
         begin
            if (pong_valid == 1'b0)
            begin
               pong_d <= d ; 
               pong_a <= a ; 
               pong_we <= we ; 
            end 
         end 
      end  
   end 
   //
   // multiplex pingpong data to pp_d, pp_a, pp_we
   assign pp_d = (rpp == 1'b1) ? ping_d : pong_d ;
   assign pp_a = (rpp == 1'b1) ? ping_a : pong_a ;
   assign pp_we = (rpp == 1'b1) ? ping_we : pong_we ;
   //
   // generate PIOreq
   assign PIOreq = (rpp == 1'b1) ? (ping_valid & ~idone) : (pong_valid & ~idone) ;
   //
   // Hookup PIO access controller
   //

   atahost_pio_actrl #(TWIDTH, PIO_mode0_T1, PIO_mode0_T2, PIO_mode0_T4, PIO_mode0_Teoc) PIO_access_control(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .IDEctrl_FATR0(IDEctrl_FATR0), 
      .IDEctrl_FATR1(IDEctrl_FATR1), 
      .cmdport_T1(cmdport_T1), 
      .cmdport_T2(cmdport_T2), 
      .cmdport_T4(cmdport_T4), 
      .cmdport_Teoc(cmdport_Teoc), 
      .cmdport_IORDYen(cmdport_IORDYen), 
      .dport0_T1(dport0_T1), 
      .dport0_T2(dport0_T2), 
      .dport0_T4(dport0_T4), 
      .dport0_Teoc(dport0_Teoc), 
      .dport0_IORDYen(dport0_IORDYen), 
      .dport1_T1(dport1_T1), 
      .dport1_T2(dport1_T2), 
      .dport1_T4(dport1_T4), 
      .dport1_Teoc(dport1_Teoc), 
      .dport1_IORDYen(dport1_IORDYen), 
      .SelDev(SelDev), 
      .go(go), 
      .done(idone), 
      .dir(pp_we), 
      .a(pp_a), 
      .q(q), 
      .DDi(DDi), 
      .oe(DDoe), 
      .DIOR(DIOR), 
      .DIOW(DIOW), 
      .IORDY(IORDY)
   ); 
//
// assign outputs
//
assign PIOa = pp_a ;
assign PIOd = pp_d ;
assign done = idone ;
endmodule
