//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores ATA/ATAPI-5 Host Controller                      ----
//--  PIO Timing Controller (common for all OCIDEC cores)        ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
// rev.: 1.0 march  7th, 2001. Initial release
// rev.: 1.1 July  11th, 2001. Changed 'igo' & 'hold_go' signal generation.
//
//
//  CVS Log
//
//  $Id: atahost_pio_tctrl.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_pio_tctrl.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
//
//-------------------------
// PIO Timing controller --
//-------------------------
//
//
// Timing	PIO mode transfers
//--------------------------------------------
// T0:	cycle time
// T1:	address valid to DIOR-/DIOW-
// T2:	DIOR-/DIOW- pulse width
// T2i:	DIOR-/DIOW- recovery time
// T3:	DIOW- data setup
// T4:	DIOW- data hold
// T5:	DIOR- data setup
// T6:	DIOR- data hold
// T9:	address hold from DIOR-/DIOW- negated
// Trd:	Read data valid to IORDY asserted
// Ta:	IORDY setup time
// Tb:	IORDY pulse width
//
// Transfer sequence
//--------------------------------
// 1)	set address (DA, CS0-, CS1-)
// 2)	wait for T1
// 3)	assert DIOR-/DIOW-
//	   when write action present Data (timing spec. T3 always honored), enable output enable-signal
// 4)	wait for T2
// 5)	check IORDY
//	   when not IORDY goto 5
// 	  when IORDY negate DIOW-/DIOR-, latch data (if read action)
//    when write, hold data for T4, disable output-enable signal
// 6)	wait end_of_cycle_time. This is T2i or T9 or (T0-T1-T2) whichever takes the longest
// 7)	start new cycle
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_pio_tctrl (clk, nReset, rst, IORDY_en, T1, T2, T4, Teoc, go, we, oe, done, dstrb, DIOR, DIOW, IORDY);

    // counter width
   parameter TWIDTH = 8;
    // 70ns
   parameter PIO_mode0_T1 = 6;
    // 290ns
   parameter PIO_mode0_T2 = 28;
    // 30ns
   parameter PIO_mode0_T4 = 2;
   parameter PIO_mode0_Teoc  = 23;
   input clk; // master clock
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input IORDY_en; // use IORDY (or not)
   input[TWIDTH - 1:0] T1; // T1 time (in clk-ticks)
   input[TWIDTH - 1:0] T2; // T2 time (in clk-ticks)
   input[TWIDTH - 1:0] T4; // T4 time (in clk-ticks)
   input[TWIDTH - 1:0] Teoc; // end of cycle time
   input go; // PIO controller selected (strobe signal)
   input we; // write enable signal. '0'=read from device, '1'=write to device
   output oe; // output enable signal
   reg oe;
   output done; // finished cycle
   wire done;
   output dstrb; // data strobe, latch data (during read)
   reg dstrb;
   output DIOR; // IOwrite signal, active high
   reg DIOR;
   output DIOW; // IOwrite signal, active high
   reg DIOW;
   input IORDY; 

   // initial data after reset
   // current counter value
   wire T1done; 
   wire T2done; 
   wire T4done; 
   wire Teoc_done; 
   wire IORDY_done; 
   reg busy; 
   reg hold_go; 
   wire igo; 
   reg hT2done; 

   // generate internal go strobe
   // strecht go until ready for new cycle
   always @(posedge clk or negedge nReset)
   begin
      if (nReset == 1'b0)
      begin
         busy <= 1'b0 ; 
         hold_go <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            busy <= 1'b0 ; 
            hold_go <= 1'b0 ; 
         end
         else
         begin
            busy <= (igo | busy) & ~Teoc_done ; 
            hold_go <= (go | (hold_go & busy)) & ~igo ; 
         end 
      end 
   end 
   assign igo = (go | hold_go) & ~busy ;
   // 1)	hookup T1 counter

 
   ro_cnt #(.SIZE(TWIDTH),     
.UD(1'b0),        
.ID(PIO_mode0_T1)
) t1_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(igo), 
      .d(T1), 
      .cnt_en(1'b1),
      .q(),
      .done(T1done)
   ); 

 
   always @(posedge clk or negedge nReset)
   begin : T2proc
      if (nReset == 1'b0)
      begin
         DIOR <= 1'b0 ; 
         DIOW <= 1'b0 ; 
         oe <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            DIOR <= 1'b0 ; 
            DIOW <= 1'b0 ; 
            oe <= 1'b0 ; 
         end
         else
         begin
            DIOR <= (~we & T1done) | (DIOR & ~IORDY_done) ; 
            DIOW <= (we & T1done) | (DIOW & ~IORDY_done) ; 
            oe <= ((we & igo) | oe) & ~T4done ; // negate oe when t4-done
         end 
      end 
   end 
   // 3)	hookup T2 counter

   ro_cnt #(.SIZE(TWIDTH),     
.UD(1'b0),        
.ID(PIO_mode0_T2)
) t2_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(T1done), 
      .d(T2),
      .cnt_en(1'b1),
      .q(), 
      .done(T2done)
   ); 

   // 4)	check IORDY (if used), generate release_DIOR-/DIOW- signal (ie negate DIOR-/DIOW-)
   // hold T2done
   always @(posedge clk or negedge nReset)
   begin : gen_hT2done
      if (nReset == 1'b0)
      begin
         hT2done <= 1'b0 ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            hT2done <= 1'b0 ; 
         end
         else
         begin
            hT2done <= (T2done | hT2done) & ~IORDY_done ; 
         end 
      end 
   end 
   assign IORDY_done = (T2done | hT2done) & (IORDY | ~IORDY_en) ;

   // generate datastrobe, capture data at rising DIOR- edge
   always @(posedge clk)
   begin : gen_dstrb
      dstrb <= IORDY_done ;  
   end 
   // hookup data hold counter


   ro_cnt #(.SIZE(TWIDTH),     
.UD(1'b0),        
.ID(PIO_mode0_T4)
) dhold_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(IORDY_done), 
      .d(T4),
      .cnt_en(1'b1),
      .q(), 
      .done(T4done)
   ); 
   assign done = T4done ;
   // while still guaranteeing data and address hold-times
   // 5)	hookup end_of_cycle counter


   ro_cnt #(.SIZE(TWIDTH),
.UD(1'b0),        
.ID(PIO_mode0_Teoc)
) eoc_cnt(
      .clk(clk), 
      .nReset(nReset), 
      .rst(rst), 
      .go(IORDY_done), 
      .d(Teoc), 
      .cnt_en(1'b1),
      .q(),
      .done(Teoc_done)
   ); 
endmodule
