//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  ATA/ATAPI-5 IDE controller with DMA support                ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2002 Richard Herveille                        ----
//--                    richard@asics.ws                         ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
//
// rev.:	1.0  Alpha version Januar 1st, 2001
// rev.: 1.0a Removed all references to records.vhd. Make core compatible with VHDL to Verilog translator tools
//            Changed DMA_req signal generation. Make the core compatible with the latest version of the OpenCores DMA engine
// rev.: 1.1  june 18th, 2001. Changed wishbone address-input from ADR_I(4 downto 0) to ADR(6 downto 2)
// rev.: 1.1a june 19th, 2001. Simplified DAT_O output multiplexor
//
//  CVS Log
//
//  $Id: atahost_top.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_top.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
// DeviceType: OCIDEC-3: OpenCores IDE Controller type3
// Features: PIO Compatible Timing, PIO Fast Timing 0/1, Single/Multiword DMA Timing 0/1
// DeviceID: 0x03
// RevNo : 0x00
//
// Host signals:
// Reset
// DIOR-		read strobe. The falling edge enables data from device onto DD. The rising edge latches data at the host.
// DIOW-		write strobe. The rising edge latches data from DD into the device.
// DMACK-	DMA acknowledge
// DA(2:0) 3bit binary coded adress
// CS0-		 select command block registers
// CS1-		 select control block registers
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module atahost_top (wb_clk_i, arst_i, wb_rst_i, wb_cyc_i, wb_stb_i, wb_ack_o, wb_rty_o, wb_err_o,TxbufQ, wb_adr_i, wb_dat_i, wb_dat_o, wb_sel_i, wb_we_i, wb_inta_o, DMA_req, DMA_Ack, resetn_pad_o, dd_pad_i, dd_pad_o, dd_padoe_o, da_pad_o, cs0n_pad_o, cs1n_pad_o, diorn_pad_o, diown_pad_o, iordy_pad_i, intrq_pad_i, dmarq_pad_i, dmackn_pad_o,DMAq);

  
   input wb_clk_i; // master clock in
   input arst_i; // asynchronous active low reset
   input wb_rst_i; // synchronous active high reset
   input wb_cyc_i; // valid bus cycle input
   input wb_stb_i; // strobe/core select input
   output wb_ack_o; // strobe acknowledge output
   wire wb_ack_o;
   output wb_rty_o; // retry output
   wire wb_rty_o;
   output wb_err_o; // error output
   wire wb_err_o;
   input[6:2] wb_adr_i; // A6 = '1' ATA devices selected
   input[31:0] wb_dat_i; // Databus in
   output[31:0] wb_dat_o; // Databus out
   wire[31:0] wb_dat_o,TxbufQ;
   input[3:0] wb_sel_i; // Byte select signals
   input wb_we_i; // Write enable input
   output wb_inta_o; // interrupt request signal IDE0
   wire wb_inta_o;
   output DMA_req; // DMA request
   wire DMA_req;
   input DMA_Ack; // DMA acknowledge
   output resetn_pad_o; 
   wire resetn_pad_o;
   input[15:0] dd_pad_i; 
   output[15:0] dd_pad_o; 
   wire[15:0] dd_pad_o;
   output[31:0]TxbufQ;
   output dd_padoe_o; 
   wire dd_padoe_o;
   output[2:0] da_pad_o; 
   wire[2:0] da_pad_o;
   output cs0n_pad_o; 
   wire cs0n_pad_o;
   output cs1n_pad_o; 
   wire cs1n_pad_o;
   output diorn_pad_o; 
   wire diorn_pad_o;
   output diown_pad_o; 
   wire diown_pad_o;
   input iordy_pad_i; 
   input intrq_pad_i; 
   input dmarq_pad_i; 
   output dmackn_pad_o; 
   wire dmackn_pad_o;
 output [31:0]DMAq;
   parameter ARST_LVL = 1'b0;		 // asynchronous reset level   
   parameter TWIDTH = 8;	 // counter width
    
   parameter PIO_mode0_T1 = 6;// 70ns    
   parameter PIO_mode0_T2 = 28;// 290ns    
   parameter PIO_mode0_T4 = 2;// 30ns
	 parameter PIO_mode0_Teoc = 23;    // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
    
   parameter DMA_mode0_Tm = 4;	// 50ns   
   parameter DMA_mode0_Td = 21;	 // 215ns
   parameter DMA_mode0_Teoc  = 21;
   //
   // component declarations
   //
   // PIO mode 0 settings (@100MHz clock)
   // Multiword DMA mode 0 settings (@100MHz clock)
   // 215ns ==> T0 - Td - Tm = 480 - 50 - 215 = 215
   // WISHBONE SYSCON signals
   // WISHBONE SLAVE signals
   //          A5 = '1' CS1- asserted, '0' CS0- asserted
   //          A4..A2 ATA address lines
   // A6 = '0' ATA controller selected
   // PIO control input
   // PIO transfer in progress
   // DMA control inputs
   // DMA transfer in progress
   // DMA receive buffer empty
   // DMA transmit buffer full
   // outputs
   // control register outputs
   // CMD port timing registers
   // data-port0 timing registers
   // data-port1 timing registers
   // DMA device0 timing registers
   // DMA device1 timing registers
   //
   // PIO mode 0 settings (@100MHz clock)
   // Multiword DMA mode 0 settings (@100MHz clock)
   // 215ns ==> T0 - Td - Tm = 480 - 50 - 215 = 215
   // control / registers
   // PIO registers
   // DMA registers
   // ATA signals
   // asynchronous reset signal
   wire arst_signal; 
   // primary address decoder
   wire PIOsel; // controller select, IDE devices select
   wire DMAsel; // controller select, IDE devices select
   // registers
   // IDE control register
   wire IDEctrl_IDEen; 
   wire IDEctrl_rst; 
   wire IDEctrl_ppen; 
   wire IDEctrl_FATR0; 
   wire IDEctrl_FATR1; 
   // PIO compatible timing settings
   wire[7:0] PIO_cmdport_T1; 
   wire[7:0] PIO_cmdport_T2; 
   wire[7:0] PIO_cmdport_T4; 
   wire[7:0] PIO_cmdport_Teoc; 
   wire PIO_cmdport_IORDYen; 
   // PIO data register device0 timing settings
   wire[7:0] PIO_dport0_T1; 
   wire[7:0] PIO_dport0_T2; 
   wire[7:0] PIO_dport0_T4; 
   wire[7:0] PIO_dport0_Teoc; 
   wire PIO_dport0_IORDYen; 
   // PIO data register device1 timing settings
   wire[7:0] PIO_dport1_T1; 
   wire[7:0] PIO_dport1_T2; 
   wire[7:0] PIO_dport1_T4; 
   wire[7:0] PIO_dport1_Teoc; 
   wire PIO_dport1_IORDYen; 
   // DMA control register
   wire DMActrl_DMAen; 
   wire DMActrl_dir; 
   wire DMActrl_BeLeC0; 
   wire DMActrl_BeLeC1; 
   // DMA data port device0 timing settings
   wire[7:0] DMA_dev0_Td; 
   wire[7:0] DMA_dev0_Tm; 
   wire[7:0] DMA_dev0_Teoc; 
   // DMA data port device1 timing settings
   wire[7:0] DMA_dev1_Td; 
   wire[7:0] DMA_dev1_Tm; 
   wire[7:0] DMA_dev1_Teoc; 
   wire PIOack; 
   wire DMAack; 
   wire PIOtip; 
   wire DMAtip; 
   wire[15:0] PIOq; 
   wire PIOpp_full; 
   wire[31:0] DMAq; 
   wire DMA_dmarq; // synchronized version of DMARQ
   wire DMATxFull; 
   wire DMARxEmpty; 
   wire irq; // ATA bus IRQ signal

   // generate asynchronous reset level
   assign arst_signal = arst_i ^ ARST_LVL ;
   //
   // hookup wishbone slave
   //
   // PIO mode 0 settings
   // Multiword DMA mode 0 settings
   // OCIDEC-1 does not support DMA, set registers to zero
   // WISHBONE SYSCON signals
   // WISHBONE SLAVE signals
   // PIO control inputs
   // DMA control inputs
   // outputs
   // control register outputs
   // CMD port timing registers
   // data-port0 timing registers
   // data-port1 timing registers
   // DMA device0 timing registers
   // DMA device1 timing registers
parameter DeviceId=4'd3  ;
parameter RevisionNo=4'd0;
  
   atahost_wb_slave #(
   								DeviceId, 
									RevisionNo, 
									PIO_mode0_T1,
									PIO_mode0_T2, 
									PIO_mode0_T4, 
									PIO_mode0_Teoc, 
									0, 
									0, 
									0
   							) 
   				u0(
      .clk_i(wb_clk_i), 
      .arst_i(arst_signal), 
      .rst_i(wb_rst_i), 
      .cyc_i(wb_cyc_i), 
      .stb_i(wb_stb_i), 
      .ack_o(wb_ack_o), 
      .rty_o(wb_rty_o), 
      .err_o(wb_err_o), 
      .adr_i(wb_adr_i), 
      .dat_i(wb_dat_i), 
      .dat_o(wb_dat_o), 
      .sel_i(wb_sel_i), 
      .we_i(wb_we_i), 
      .inta_o(wb_inta_o), 
      .PIOsel(PIOsel), 
      .PIOtip(PIOtip), 
      .PIOack(PIOack), 
      .PIOq(PIOq), 
      .PIOpp_full(PIOpp_full), 
      .irq(irq), //    
      .DMAsel(DMAsel), 
      .DMAtip(DMAtip), //   
      .DMAack(DMAack), 
      .DMARxEmpty(DMARxEmpty), 
      .DMATxFull(DMATxFull), 
      .DMA_dmarq(DMA_dmarq), 
      .DMAq(DMAq), 
      .IDEctrl_rst(IDEctrl_rst), 
      .IDEctrl_IDEen(IDEctrl_IDEen), 
      .IDEctrl_FATR0(IDEctrl_FATR0), 
      .IDEctrl_FATR1(IDEctrl_FATR1), 
      .IDEctrl_ppen(IDEctrl_ppen), // 
      .DMActrl_DMAen(DMActrl_DMAen), 
      .DMActrl_dir(DMActrl_dir), 
      .DMActrl_BeLeC0(DMActrl_BeLeC0), 
      .DMActrl_BeLeC1(DMActrl_BeLeC1), 
      .PIO_cmdport_T1(PIO_cmdport_T1), 
      .PIO_cmdport_T2(PIO_cmdport_T2), 
      .PIO_cmdport_T4(PIO_cmdport_T4), 
      .PIO_cmdport_Teoc(PIO_cmdport_Teoc), 
      .PIO_cmdport_IORDYen(PIO_cmdport_IORDYen), 
      .PIO_dport0_T1(PIO_dport0_T1), 
      .PIO_dport0_T2(PIO_dport0_T2), 
      .PIO_dport0_T4(PIO_dport0_T4), 
      .PIO_dport0_Teoc(PIO_dport0_Teoc), 
      .PIO_dport0_IORDYen(PIO_dport0_IORDYen), 
      .PIO_dport1_T1(PIO_dport1_T1), 
      .PIO_dport1_T2(PIO_dport1_T2), 
      .PIO_dport1_T4(PIO_dport1_T4), 
      .PIO_dport1_Teoc(PIO_dport1_Teoc), 
      .PIO_dport1_IORDYen(PIO_dport1_IORDYen), 
      .DMA_dev0_Tm(DMA_dev0_Tm), 
      .DMA_dev0_Td(DMA_dev0_Td), 
      .DMA_dev0_Teoc(DMA_dev0_Teoc), 
      .DMA_dev1_Tm(DMA_dev1_Tm), 
      .DMA_dev1_Td(DMA_dev1_Td), 
      .DMA_dev1_Teoc(DMA_dev1_Teoc)
   ); 
   //
   // hookup controller section
   //

   atahost_controller #(PIO_mode0_T1,   
				PIO_mode0_T2,   
				PIO_mode0_T4,   
				PIO_mode0_Teoc, 
				DMA_mode0_Tm,   
				DMA_mode0_Td,   
				DMA_mode0_Teoc  
				) u1(
      .clk(wb_clk_i), 
      .nReset(arst_signal), 
      .rst(wb_rst_i), 
      .irq(irq), 
      .IDEctrl_IDEen(IDEctrl_IDEen), //IDEctrl_IDEen
      .IDEctrl_rst(IDEctrl_rst),       //IDEctrl_rst
      .IDEctrl_ppen (IDEctrl_ppen  ),// IDEctrl_ppen
      .IDEctrl_FATR0(IDEctrl_FATR0 ),//IDEctrl_FATR0 
      .IDEctrl_FATR1(IDEctrl_FATR1 ),//IDEctrl_FATR1 
      .a(wb_adr_i[5:2]), 
      .d(wb_dat_i), 
      .we(wb_we_i), 
      .PIO_cmdport_T1(PIO_cmdport_T1),//PIO_cmdport_T1 
      .PIO_cmdport_T2(PIO_cmdport_T2),//PIO_cmdport_T2 
      .PIO_cmdport_T4(PIO_cmdport_T4),//PIO_cmdport_T4 
      .PIO_cmdport_Teoc(PIO_cmdport_Teoc), //PIO_cmdport_Teoc
      .PIO_cmdport_IORDYen(PIO_cmdport_IORDYen), //PIO_cmdport_IORDYen
      .PIO_dport0_T1(PIO_dport0_T1), //PIO_dport0_T1
      .PIO_dport0_T2(PIO_dport0_T2), //PIO_dport0_T2
      .PIO_dport0_T4(PIO_dport0_T4), //PIO_dport0_T4
      .PIO_dport0_Teoc(PIO_dport0_Teoc), //PIO_dport0_Teoc
      .PIO_dport0_IORDYen(PIO_dport0_IORDYen), //PIO_dport0_IORDYen
      .PIO_dport1_T1(PIO_dport1_T1), //PIO_dport1_T1
      .PIO_dport1_T2(PIO_dport1_T2), //PIO_dport1_T2
      .PIO_dport1_T4(PIO_dport1_T4), //PIO_dport1_T4
      .PIO_dport1_Teoc(PIO_dport1_Teoc), //PIO_dport1_Teoc
      .PIO_dport1_IORDYen(PIO_dport1_IORDYen), //PIO_dport1_IORDYen
      .PIOsel(PIOsel), //PIOsel
      .PIOack(PIOack), 
      .PIOq(PIOq), 
      .TxbufQ(TxbufQ),
      .PIOtip(PIOtip), 
      .PIOpp_full(PIOpp_full), 
      .DMActrl_DMAen(DMActrl_DMAen), 			//DMActrl_DMAen
      .DMActrl_dir(DMActrl_dir), //DMActrl_dir
      .DMActrl_BeLeC0(DMActrl_BeLeC0),//DMActrl_BeLeC0 
      .DMActrl_BeLeC1(DMActrl_BeLeC1),//DMActrl_BeLeC1 
      .DMA_dev0_Td(DMA_dev0_Td),//DMA_dev0_Td 
      .DMA_dev0_Tm(DMA_dev0_Tm),//DMA_dev0_Tm 
      .DMA_dev0_Teoc(DMA_dev0_Teoc),//DMA_dev0_Teoc 
      .DMA_dev1_Td(DMA_dev1_Td), //DMA_dev1_Td
      .DMA_dev1_Tm(DMA_dev1_Tm), //DMA_dev1_Tm
      .DMA_dev1_Teoc(DMA_dev1_Teoc), //DMA_dev1_Teoc
      .DMAsel(DMAsel), //DMAsel
      .DMAack(DMAack),//DMAack 
      .DMAq(DMAq), //DMAq
      .DMAtip(DMAtip),  //DMAtip
      .DMA_dmarq(DMA_dmarq),  //DMA_dmarq
      .DMATxFull(DMATxFull),  //DMATxFull
      .DMARxEmpty(DMARxEmpty), //DMARxEmpty
      .DMA_req(DMA_req), //DMA_req
      .DMA_ack(DMA_Ack), //DMA_Ack
      .RESETn(resetn_pad_o), //resetn_pad_o
      .DDi(dd_pad_i), 
      .DDo(dd_pad_o), 
      .DDoe(dd_padoe_o), 
      .DA(da_pad_o), 
      .CS0n(cs0n_pad_o), 
      .CS1n(cs1n_pad_o), 
      .DIORn(diorn_pad_o), 
      .DIOWn(diown_pad_o), 
      .IORDY(iordy_pad_i), 
      .INTRQ(intrq_pad_i), 
      .DMARQ(dmarq_pad_i), //
      .DMACKn(dmackn_pad_o)
   ); 
endmodule
