//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//-------------------------------------------------------------------
//--                                                             ----
//--  OpenCores IDE Controller                                   ----
//--  Wishbone Slave (common for all OCIDEC cores)               ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2002 Richard Herveille                        ----
//--                    richard@asics.ws                         ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
//
//  CVS Log
//
//  $Id: atahost_wb_slave.vhd,v 1.1 2002/02/18 14:32:12 rherveille Exp $
//
//  $Date: 2002/02/18 14:32:12 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: atahost_wb_slave.vhd,v $
//               Revision 1.1  2002/02/18 14:32:12  rherveille
//               renamed all files to 'atahost_***.vhd'
//               broke-up 'counter.vhd' into 'ud_cnt.vhd' and 'ro_cnt.vhd'
//               changed resD input to generic RESD in ud_cnt.vhd
//               changed ID input to generic ID in ro_cnt.vhd
//               changed core to reflect changes in ro_cnt.vhd
//               removed references to 'count' library
//               changed IO names
//               added disclaimer
//               added CVS log
//               moved registers and wishbone signals into 'atahost_wb_slave.vhd'
//
//
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
`define ATA_DEV_ADR adr_i[6]
`define ATA_ADR adr_i[5:2]
//
// constants
//
// addresses
`define ATA_CTRL_REG 4'b0000
`define ATA_STAT_REG 4'b0001
`define ATA_PIO_CMD 4'b0010
`define ATA_PIO_DP0 4'b0011
`define ATA_PIO_DP1 4'b0100
`define ATA_DMA_DEV0 4'b0101
`define ATA_DMA_DEV1 4'b0110
// reserved --
`define ATA_DMA_PORT 4'b1111
module atahost_wb_slave (clk_i, arst_i, rst_i, cyc_i, stb_i, ack_o, rty_o, err_o, adr_i, dat_i, dat_o, sel_i, we_i, inta_o, PIOsel, PIOtip, PIOack, PIOq, PIOpp_full, irq, DMAsel, DMAtip, DMAack, DMARxEmpty, DMATxFull, DMA_dmarq, DMAq, IDEctrl_rst, IDEctrl_IDEen, IDEctrl_FATR1, IDEctrl_FATR0, IDEctrl_ppen, DMActrl_DMAen, DMActrl_dir, DMActrl_BeLeC0, DMActrl_BeLeC1, PIO_cmdport_T1, PIO_cmdport_T2, PIO_cmdport_T4, PIO_cmdport_Teoc, PIO_cmdport_IORDYen, PIO_dport0_T1, PIO_dport0_T2, PIO_dport0_T4, PIO_dport0_Teoc, PIO_dport0_IORDYen, PIO_dport1_T1, PIO_dport1_T2, PIO_dport1_T4, PIO_dport1_Teoc, PIO_dport1_IORDYen, DMA_dev0_Tm, DMA_dev0_Td, DMA_dev0_Teoc, DMA_dev1_Tm, DMA_dev1_Td, DMA_dev1_Teoc);

   parameter[3:0] DeviceID  = 0;
   parameter[3:0] RevisionNo  =0;
    // 70ns
   parameter PIO_mode0_T1 = 6;
    // 290ns
   parameter PIO_mode0_T2 = 28;
    // 30ns
   parameter PIO_mode0_T4 = 2;
    // 240ns ==> T0 - T1 - T2 = 600 - 70 - 290 = 240
   parameter PIO_mode0_Teoc = 23;
    // 50ns
   parameter DMA_mode0_Tm = 4;
    // 215ns
   parameter DMA_mode0_Td = 21;
   parameter DMA_mode0_Teoc  = 21;
   input clk_i; // master clock in
   input arst_i; // asynchronous active low reset
   input rst_i; // synchronous active high reset
   input cyc_i; // valid bus cycle input
   input stb_i; // strobe/core select input
   output ack_o; // strobe acknowledge output
   wire ack_o;
   output rty_o; // retry output
   wire rty_o;
   output err_o; // error output
   wire err_o;
   input[6:2] adr_i; // A6 = '1' ATA devices selected
   input[31:0] dat_i; // Databus in
   output[31:0] dat_o; // Databus out
   wire[31:0] dat_o;
   input[3:0] sel_i; // Byte select signals
   input we_i; // Write enable input
   output inta_o; // interrupt request signal IDE0
   wire inta_o;
   output PIOsel; 
   wire PIOsel;
   input PIOtip; // PIO acknowledge signal
   input PIOack; // PIO acknowledge signal
   input[15:0] PIOq; // PIO data input
   input PIOpp_full; // PIO write-ping-pong buffers full
   input irq; // interrupt signal input
   output DMAsel; 
   wire DMAsel;
   input DMAtip; // wishbone DMA request
   input DMAack; // wishbone DMA request
   input DMARxEmpty; // wishbone DMA request
   input DMATxFull; // wishbone DMA request
   input DMA_dmarq; // wishbone DMA request
   input[31:0] DMAq; 
   output IDEctrl_rst; 
   wire IDEctrl_rst;
   output IDEctrl_IDEen; 
   wire IDEctrl_IDEen;
   output IDEctrl_FATR1; 
   wire IDEctrl_FATR1;
   output IDEctrl_FATR0; 
   wire IDEctrl_FATR0;
   output IDEctrl_ppen; 
   wire IDEctrl_ppen;
   output DMActrl_DMAen; 
   wire DMActrl_DMAen;
   output DMActrl_dir; 
   wire DMActrl_dir;
   output DMActrl_BeLeC0; 
   wire DMActrl_BeLeC0;
   output DMActrl_BeLeC1; 
   wire DMActrl_BeLeC1;
   output[7:0] PIO_cmdport_T1; 
   reg[7:0] PIO_cmdport_T1;
   output[7:0] PIO_cmdport_T2; 
   reg[7:0] PIO_cmdport_T2;
   output[7:0] PIO_cmdport_T4; 
   reg[7:0] PIO_cmdport_T4;
   output[7:0] PIO_cmdport_Teoc; 
   reg[7:0] PIO_cmdport_Teoc;
   output PIO_cmdport_IORDYen; 
   wire PIO_cmdport_IORDYen;
   output[7:0] PIO_dport0_T1; 
   reg[7:0] PIO_dport0_T1;
   output[7:0] PIO_dport0_T2; 
   reg[7:0] PIO_dport0_T2;
   output[7:0] PIO_dport0_T4; 
   reg[7:0] PIO_dport0_T4;
   output[7:0] PIO_dport0_Teoc; 
   reg[7:0] PIO_dport0_Teoc;
   output PIO_dport0_IORDYen; 
   wire PIO_dport0_IORDYen;
   output[7:0] PIO_dport1_T1; 
   reg[7:0] PIO_dport1_T1;
   output[7:0] PIO_dport1_T2; 
   reg[7:0] PIO_dport1_T2;
   output[7:0] PIO_dport1_T4; 
   reg[7:0] PIO_dport1_T4;
   output[7:0] PIO_dport1_Teoc; 
   reg[7:0] PIO_dport1_Teoc;
   output PIO_dport1_IORDYen; 
   wire PIO_dport1_IORDYen;
   output[7:0] DMA_dev0_Tm; 
   reg[7:0] DMA_dev0_Tm;
   output[7:0] DMA_dev0_Td; 
   reg[7:0] DMA_dev0_Td;
   output[7:0] DMA_dev0_Teoc; 
   reg[7:0] DMA_dev0_Teoc;
   output[7:0] DMA_dev1_Tm; 
   reg[7:0] DMA_dev1_Tm;
   output[7:0] DMA_dev1_Td; 
   reg[7:0] DMA_dev1_Td;
   output[7:0] DMA_dev1_Teoc; 
   reg[7:0] DMA_dev1_Teoc;


   //
   // primary address decoder
   wire CONsel; // controller select, IDE devices select
   wire berr; // bus error, bus retry
   wire brty; // bus error, bus retry
   // registers
   reg[31:0] CtrlReg; // control and status registers
   reg[31:0] StatReg; // control and status registers

   //
   //
   // generate bus cycle / address decoder
   //
   wire w_acc; // word access, double word access
   wire dw_acc; // word access, double word access
   reg store_pp_full; 
   // word / double word
   assign w_acc = sel_i[1] & sel_i[0] ;
   assign dw_acc = sel_i[3] & sel_i[2] & sel_i[1] & sel_i[0] ;
   // bus error
   assign berr = (`ATA_DEV_ADR == 1'b1) ? ~w_acc : ~dw_acc ;
   // PIO accesses at least 16bit wide, no PIO access during DMAtip or pingpong full
   assign PIOsel = cyc_i & stb_i & `ATA_DEV_ADR & w_acc & ~(DMAtip | store_pp_full) ;
   // CON accesses only 32bit wide	-- means controller is selected
   assign CONsel = cyc_i & stb_i & ~`ATA_DEV_ADR & dw_acc ;
   assign DMAsel = CONsel & (`ATA_ADR == `ATA_DMA_PORT) ;

   // bus retry (OCIDEC-3 and above)
   // store PIOpp_full, we don't want a PPfull based retry initiated by the current bus-cycle
   always @(posedge clk_i)
   begin
      if (PIOsel == 1'b0)
      begin
         store_pp_full <= PIOpp_full ; 
      end  
   end 
   assign brty = (`ATA_DEV_ADR & w_acc) & (DMAtip | store_pp_full) ;
   //
   // generate registers
   //
   wire sel_PIO_cmdport; // PIO timing registers
   wire sel_PIO_dport0; // PIO timing registers
   wire sel_PIO_dport1; // PIO timing registers
   wire sel_DMA_dev0; // DMA timing registers
   wire sel_DMA_dev1; // DMA timing registers
   wire sel_ctrl; // control / status register
   wire sel_stat; // control / status register
   // generate register select signals
   assign sel_ctrl = CONsel & we_i & (`ATA_ADR == `ATA_CTRL_REG) ;
   assign sel_stat = CONsel & we_i & (`ATA_ADR == `ATA_STAT_REG) ;
   assign sel_PIO_cmdport = CONsel & we_i & (`ATA_ADR == `ATA_PIO_CMD) ;
   assign sel_PIO_dport0 = CONsel & we_i & (`ATA_ADR == `ATA_PIO_DP0) ;
   assign sel_PIO_dport1 = CONsel & we_i & (`ATA_ADR == `ATA_PIO_DP1) ;
   assign sel_DMA_dev0 = CONsel & we_i & (`ATA_ADR == `ATA_DMA_DEV0) ;
   assign sel_DMA_dev1 = CONsel & we_i & (`ATA_ADR == `ATA_DMA_DEV1) ;

   // reserved 0x1C-0x38 --
   // reserved 0x3C : DMA port --
   // generate control register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_ctrl_reg
      if (arst_i == 1'b0)
      begin
         CtrlReg[31:1] <= {31{1'b0}} ; 
         CtrlReg[0] <= 1'b1 ; // set reset bit
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            CtrlReg[31:1] <= {31{1'b0}} ; 
            CtrlReg[0] <= 1'b1 ; // set reset bit
         end
         else if (sel_ctrl == 1'b1)
         begin
            CtrlReg <= dat_i ; 
         end 
      end 
   end 
   // assign bits
   assign DMActrl_DMAen = CtrlReg[15] ;
   assign DMActrl_dir = CtrlReg[13] ;
   assign DMActrl_BeLeC1 = CtrlReg[9] ;
   assign DMActrl_BeLeC0 = CtrlReg[8] ;
   assign IDEctrl_IDEen = CtrlReg[7] ;
   assign IDEctrl_FATR1 = CtrlReg[6] ;
   assign IDEctrl_FATR0 = CtrlReg[5] ;
   assign IDEctrl_ppen = CtrlReg[4] ;
   assign PIO_dport1_IORDYen = CtrlReg[3] ;
   assign PIO_dport0_IORDYen = CtrlReg[2] ;
   assign PIO_cmdport_IORDYen = CtrlReg[1] ;
   assign IDEctrl_rst = CtrlReg[0] ;
   // generate status register clearable bits
   reg dirq; 
   reg int; 

   always @(posedge clk_i or negedge arst_i)
   begin : gen_irq
      if (arst_i == 1'b0)
      begin
         int <= 1'b0 ; 
         dirq <= 1'b0 ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            int <= 1'b0 ; 
            dirq <= 1'b0 ; 
         end
         else
         begin
            int <= (int | (irq & ~dirq)) & ~(sel_stat & ~dat_i[0]) ; 
            dirq <= irq ; 
         end 
      end 
   end 

   always @(arst_i or rst_i or DMAtip or DMARxEmpty or DMATxFull or DMA_dmarq or PIOtip or int or 
            PIOpp_full)
   begin : gen_stat
      //StatReg[31:0] = {32{1'b0}} ; // clear all bits (read unused bits as '0')
      StatReg[31:28] = DeviceID ; // set Device ID
      StatReg[27:24] = RevisionNo ; // set revision number
      StatReg[15] = DMAtip ; 
      StatReg[10] = DMARxEmpty ; 
      StatReg[9] = DMATxFull ; 
      StatReg[8] = DMA_dmarq ; 
      StatReg[7] = PIOtip ; 
      StatReg[6] = PIOpp_full ; 
      StatReg[0] = int ; 
      StatReg[23:16] = 8'b0;
			StatReg[14:11] = 4'b0;      
			StatReg[5:1] = 5'b0;      			
   end  
   
reg[31:0]  StatReg_reg,DMAq_reg;
always @(posedge clk_i or negedge arst_i)           
begin
		if( ~arst_i)
		begin
		   	StatReg_reg <= 32'h0; 
		   	DMAq_reg <= 32'h0;
		end
		else
		begin
				StatReg_reg <= StatReg;
				//DMAq_reg <= DMAq;
		end
end           
           
           
   // generate PIO compatible / command-port timing register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_PIO_cmdport_reg
      if (arst_i == 1'b0)
      begin
         PIO_cmdport_T1 <= PIO_mode0_T1 ; 
         PIO_cmdport_T2 <= PIO_mode0_T2 ; 
         PIO_cmdport_T4 <= PIO_mode0_T4 ; 
         PIO_cmdport_Teoc <= PIO_mode0_Teoc ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            PIO_cmdport_T1 <= PIO_mode0_T1 ; 
            PIO_cmdport_T2 <= PIO_mode0_T2 ; 
            PIO_cmdport_T4 <= PIO_mode0_T4 ; 
            PIO_cmdport_Teoc <= PIO_mode0_Teoc ; 
         end
         else if (sel_PIO_cmdport == 1'b1)
         begin
            PIO_cmdport_T1 <= dat_i[7:0] ; 
            PIO_cmdport_T2 <= dat_i[15:8] ; 
            PIO_cmdport_T4 <= dat_i[23:16] ; 
            PIO_cmdport_Teoc <= dat_i[31:24] ; 
         end 
      end 
   end 

   // generate PIO device0 timing register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_PIO_dport0_reg
      if (arst_i == 1'b0)
      begin
         PIO_dport0_T1 <= PIO_mode0_T1 ; 
         PIO_dport0_T2 <= PIO_mode0_T2 ; 
         PIO_dport0_T4 <= PIO_mode0_T4 ; 
         PIO_dport0_Teoc <= PIO_mode0_Teoc ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            PIO_dport0_T1 <= PIO_mode0_T1 ; 
            PIO_dport0_T2 <= PIO_mode0_T2 ; 
            PIO_dport0_T4 <= PIO_mode0_T4 ; 
            PIO_dport0_Teoc <= PIO_mode0_Teoc ; 
         end
         else if (sel_PIO_dport0 == 1'b1)
         begin
            PIO_dport0_T1 <= dat_i[7:0] ; 
            PIO_dport0_T2 <= dat_i[15:8] ; 
            PIO_dport0_T4 <= dat_i[23:16] ; 
            PIO_dport0_Teoc <= dat_i[31:24] ; 
         end 
      end 
   end 

   // generate PIO device1 timing register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_PIO_dport1_reg
      if (arst_i == 1'b0)
      begin
         PIO_dport1_T1 <= PIO_mode0_T1 ; 
         PIO_dport1_T2 <= PIO_mode0_T2 ; 
         PIO_dport1_T4 <= PIO_mode0_T4 ; 
         PIO_dport1_Teoc <= PIO_mode0_Teoc ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            PIO_dport1_T1 <= PIO_mode0_T1 ; 
            PIO_dport1_T2 <= PIO_mode0_T2 ; 
            PIO_dport1_T4 <= PIO_mode0_T4 ; 
            PIO_dport1_Teoc <= PIO_mode0_Teoc ; 
         end
         else if (sel_PIO_dport1 == 1'b1)
         begin
            PIO_dport1_T1 <= dat_i[7:0] ; 
            PIO_dport1_T2 <= dat_i[15:8] ; 
            PIO_dport1_T4 <= dat_i[23:16] ; 
            PIO_dport1_Teoc <= dat_i[31:24] ; 
         end 
      end 
   end 

   // generate DMA device0 timing register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_DMA_dev0_reg
      if (arst_i == 1'b0)
      begin
         DMA_dev0_Tm <= DMA_mode0_Tm ; 
         DMA_dev0_Td <= DMA_mode0_Td ; 
         DMA_dev0_Teoc <= DMA_mode0_Teoc ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            DMA_dev0_Tm <= DMA_mode0_Tm ; 
            DMA_dev0_Td <= DMA_mode0_Td ; 
            DMA_dev0_Teoc <= DMA_mode0_Teoc ; 
         end
         else if (sel_DMA_dev0 == 1'b1)
         begin
            DMA_dev0_Tm <= dat_i[7:0] ; 
            DMA_dev0_Td <= dat_i[15:8] ; 
            DMA_dev0_Teoc <= dat_i[31:24] ; 
         end 
      end 
   end 

   // generate DMA device1 timing register
   always @(posedge clk_i or negedge arst_i)
   begin : gen_DMA_dev1_reg
      if (arst_i == 1'b0)
      begin
         DMA_dev1_Tm <= DMA_mode0_Tm ; 
         DMA_dev1_Td <= DMA_mode0_Td ; 
         DMA_dev1_Teoc <= DMA_mode0_Teoc ; 
      end
      else
      begin
         if (rst_i == 1'b1)
         begin
            DMA_dev1_Tm <= DMA_mode0_Tm ; 
            DMA_dev1_Td <= DMA_mode0_Td ; 
            DMA_dev1_Teoc <= DMA_mode0_Teoc ; 
         end
         else if (sel_DMA_dev1 == 1'b1)
         begin
            DMA_dev1_Tm <= dat_i[7:0] ; 
            DMA_dev1_Td <= dat_i[15:8] ; 
            DMA_dev1_Teoc <= dat_i[31:24] ; 
         end 
      end 
   end  
   //
   // generate WISHBONE interconnect signals
   //
   reg[31:0] Q; 
   // generate acknowledge signal
   assign ack_o = PIOack | CONsel ;
   // generate error signal
   assign err_o = cyc_i & stb_i & berr ;
   // generate retry signal
   assign rty_o = cyc_i & stb_i & brty ;
   // assign interrupt signal
   assign inta_o = StatReg[0] ;
   // generate output multiplexor
   // control register
   // status register
   // PIO compatible / cmd-port timing register
   // PIO fast timing register device0
   // PIO fast timing register device1
   // DMA timing register device0
   // DMA timing register device1
   // DMA port, DMA receive register
   //assign Q = (`ATA_ADR == `ATA_CTRL_REG) ? CtrlReg : (`ATA_ADR == `ATA_STAT_REG) ? StatReg : (`ATA_ADR == `ATA_PIO_CMD) ? ({PIO_cmdport_Teoc, PIO_cmdport_T4, PIO_cmdport_T2, PIO_cmdport_T1}) : (`ATA_ADR == `ATA_PIO_DP0) ? ({PIO_dport0_Teoc, PIO_dport0_T4, PIO_dport0_T2, PIO_dport0_T1}) : (`ATA_ADR == `ATA_PIO_DP1) ? ({PIO_dport1_Teoc, PIO_dport1_T4, PIO_dport1_T2, PIO_dport1_T1}) : (`ATA_ADR == `ATA_DMA_DEV0) ? ({DMA_dev0_Teoc, 8'h0, DMA_dev0_Td, DMA_dev0_Tm}) : (`ATA_ADR == `ATA_DMA_DEV1) ? ({DMA_dev1_Teoc, 8'h00, DMA_dev1_Td, DMA_dev1_Tm}) : (`ATA_ADR == `ATA_DMA_PORT) ? DMAq : {32{1'b0}} ; 
   
   // generate output multiplexor
	always@(`ATA_ADR or CtrlReg or StatReg_reg or 
			PIO_cmdport_T1 or PIO_cmdport_T2 or PIO_cmdport_T4 or PIO_cmdport_Teoc or
			PIO_dport0_T1 or PIO_dport0_T2 or PIO_dport0_T4 or PIO_dport0_Teoc or
			PIO_dport1_T1 or PIO_dport1_T2 or PIO_dport1_T4 or PIO_dport1_Teoc or
			DMA_dev0_Tm or DMA_dev0_Td or DMA_dev0_Teoc or
			DMA_dev1_Tm or DMA_dev1_Td or DMA_dev1_Teoc or
			DMAq
		)
		case (`ATA_ADR) // synopsis full_case parallel_case
			`ATA_CTRL_REG: Q = CtrlReg;
			`ATA_STAT_REG: Q = StatReg_reg;
			`ATA_PIO_CMD : Q = {PIO_cmdport_Teoc, PIO_cmdport_T4, PIO_cmdport_T2, PIO_cmdport_T1};
			`ATA_PIO_DP0 : Q = {PIO_dport0_Teoc, PIO_dport0_T4, PIO_dport0_T2, PIO_dport0_T1};
			`ATA_PIO_DP1 : Q = {PIO_dport1_Teoc, PIO_dport1_T4, PIO_dport1_T2, PIO_dport1_T1};
			`ATA_DMA_DEV0: Q = {DMA_dev0_Teoc, 8'h0, DMA_dev0_Td, DMA_dev0_Tm};
			`ATA_DMA_DEV1: Q = {DMA_dev1_Teoc, 8'h0, DMA_dev1_Td, DMA_dev1_Tm};
			`ATA_DMA_PORT: Q = DMAq;
			 default: Q = 0;
		endcase
  
   assign dat_o = (`ATA_DEV_ADR == 1'b1) ? ({16'h0, PIOq}) : Q;
endmodule
