//--------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<
//--------------------------------------------------------------------
//-------------------------------------------------------------------
//--                                                             ----
//--  Generic Up/Down counter (ripple carry architecture)        ----
//--                                                             ----
//--  Author: Richard Herveille                                  ----
//--          richard@asics.ws                                   ----
//--          www.asics.ws                                       ----
//--                                                             ----
//-------------------------------------------------------------------
//--                                                             ----
//-- Copyright (C) 2001, 2002 Richard Herveille                  ----
//--                          richard@asics.ws                   ----
//--                                                             ----
//-- This source file may be used and distributed without        ----
//-- restriction provided that this copyright statement is not   ----
//-- removed from the file and that any derivative work contains ----
//-- the original copyright notice and the associated disclaimer.----
//--                                                             ----
//--     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     ----
//-- EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   ----
//-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   ----
//-- FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      ----
//-- OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         ----
//-- INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    ----
//-- (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   ----
//-- GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        ----
//-- BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  ----
//-- LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  ----
//-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  ----
//-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         ----
//-- POSSIBILITY OF SUCH DAMAGE.                                 ----
//--                                                             ----
//-------------------------------------------------------------------
//
//  CVS Log
//
//  $Id: ud_cnt.vhd,v 1.1 2002/03/01 03:49:25 rherveille Exp $
//
//  $Date: 2002/03/01 03:49:25 $
//  $Revision: 1.1 $
//  $Author: rherveille $
//  $Locker:  $
//  $State: Exp $
//
// Change History:
//               $Log: ud_cnt.vhd,v $
//               Revision 1.1  2002/03/01 03:49:25  rherveille
//               Changed internal counter libraries.
//               Split counter.vhd into separate files.
//               Core is in same state as Verilog version now.
//
//---------------------------------------------------------------------
// Copyright (c) 2010 by Lattice Semiconductor Corporation
//---------------------------------------------------------------------
// Disclaimer:
//
// This VHDL or Verilog source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Lattice Semiconductor provides no warranty
// regarding the use or functionality of this code.
//
// --------------------------------------------------------------------
//
// Lattice Semiconductor Corporation
// 5555 NE Moore Court
// Hillsboro, OR 97214
// U.S.A
//
// TEL: 1-800-Lattice (USA and Canada)
// 503-268-8001 (other locations)
//
// web: http://www.latticesemi.com/
// email: techsupport@latticesemi.com
//
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author     |Mod. Date |Changes Made:
// V1.0 | Peter.Zhou |06/02/10  |initial release
//
// --------------------------------------------------------------------
`timescale 1ns/1ns
module ud_cnt (clk, nReset, rst, cnt_en, ud, nld, d, q, rci, rco);

   parameter SIZE  = 8;
   parameter RESD  = 0;
   input clk; // master clock
   input nReset; // asynchronous active low reset
   input rst; // synchronous active high reset
   input cnt_en; // count enable
   input ud; // up / not down
   input nld; // synchronous active low load
   input[SIZE - 1:0] d; // load counter value
   output[SIZE - 1:0] q; // current counter value
   wire[SIZE - 1:0] q;
   input rci; // carry input
   output rco; 
   wire rco;

   reg[SIZE - 1:0] Qi; 
   wire[SIZE:0] val; 

   assign val = (ud == 1'b1) ? (({1'b0, Qi}) + rci) : (({1'b0, Qi}) - rci) ;

   always @(posedge clk or negedge nReset)
   begin : regs
      if (nReset == 1'b0)
      begin
         Qi <= RESD ; 
      end
      else
      begin
         if (rst == 1'b1)
         begin
            Qi <= RESD ; 
         end
         else
         begin
            if (nld == 1'b0)
            begin
               Qi <= d ; 
            end
            else if (cnt_en == 1'b1)
            begin
               Qi <= val[SIZE - 1:0] ; 
             
            end 
         end 
      end 
   end 
   // assign outputs
   assign q = Qi ;
 	assign rco = val[SIZE] ;
   //assign rco = Qi[SIZE] ;
endmodule
